==> spider-website-1.0.5.tgz: package/lib/file.js <==
'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * 文件对象
 *    新建对象
 *    从json中建立对象
 *    计算保存地址
 *    文件下载
 *    转换地址为相对地址
 *    触发事件
 *       init
 *       load_before
 *       loaded
 *       load_fail
 *       convert
 * */
var URL = require('url');
var isDir = require('is-dir');
var mkdir = require('mk-dir');
var request = require('request');
var fs = require('fs');
var Path = require('path');
var Hash = require('object-hash');
var _getLinks = require('ctx-links');
var convert = require('convert-relative');
var Event = require('events');
var debug = require('debug')('spider-file');
var decode = require('iconv-lite').decode;
var initState = {
  loaded: false,
  loadState: 0, // 下载状态，0：未开始，1：下载中，2： 下载完毕  3： 下载失败
  reTryTime: 0
};
var saveAttrs = ['href', 'loaded', 'data', 'reTryTime', 'hash', 'data', 'saveTo'];
var parseAttr = ['host', 'pathname', 'href', 'protocol'];

var defaultOpts = {};

var File = function (_Event) {
  _inherits(File, _Event);

  _createClass(File, null, [{
    key: 'setConfig',
    value: function setConfig(opts) {
      defaultOpts = opts;
    }
  }]);

  function File(urlORData, opts) {
    var _ret;

    _classCallCheck(this, File);

    debug('new file:', urlORData);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(File).call(this));

    _this.init(urlORData, Object.assign({}, opts, defaultOpts));
    debug('init file finished');
    return _ret = _this, _possibleConstructorReturn(_this, _ret);
  }

  _createClass(File, [{
    key: 'init',
    value: function init(data, opts) {
      if (!data) throw new Error('File.init(url,[opts]) nead url.');
      if (typeof data !== 'string' && typeof data.href !== 'string') throw new Error('File.init(url,[opts]) nead url.');
      var parse = URL.parse(typeof data === 'string' ? data : data.href);
      var me = this;
      parseAttr.forEach(function (attr) {
        me[attr] = parse[attr];
      });
      for (var attr in data) {
        if (saveAttrs.indexOf(attr) >= 0) me[attr] = data[attr] || initState[attr];
      }
      me.link = me.protocol + '//' + me.host + me.pathname;
      me.opts = opts || defaultOpts;
      if (!me.saveTo) me.saveTo = me.getSavePath();
      return me;
    }
  }, {
    key: 'toJSON',
    value: function toJSON() {
      var me = this;
      var obj = {};
      saveAttrs.forEach(function (attr) {
        obj[attr] = me[attr];
      });
      return JSON.stringify(obj);
    }
  }, {
    key: 'getSavePath',
    value: function getSavePath() {
      if (this.saveTo) return this.saveTo;
      // 判断是否需要增加扩展名
      var pathname = this.pathname || '';
      var extname = Path.extname(pathname);
      if (this.opts.autoName && (pathname[pathname.length - 1] === '/' || !extname)) {
        pathname += '/' + this.opts.autoName;
        extname = Path.extname(extname);
      }
      var index = this.opts.savePathIgnore && pathname.indexOf(this.opts.savePathIgnore);
      if (index === 0) pathname = pathname.substr(this.opts.savePathIgnore.length - 1);
      var saveTo = Path.resolve(this.opts.saveTo + '/' + pathname);
      if (isDir(saveTo)) throw new Error('saveTo path is directory...');
      return saveTo;
    }
  }, {
    key: 'load',
    value: function load() {
      var me = this;
      debug('load:', this.href);
      return new Promise(function (resolve, reject) {
        me.reTryTime = me.reTryTime + 1 || 1;
        me.loadState = 1;
        setTimeout(function () {
          debug('load timeout', me.opts.timeout || 100000);
          if (me.loadState == 1) {
            me.loadState = 3;
            me.err = new Error('timeout');
          }
          reject(me);
        }, me.opts.timeout || 100000);
        try {
          var params = Object.assign({
            url: encodeURI(me.href),
            method: 'get',
            gzip: false,
            headers: headers,
            encoding: null
          }, me.opts.request, me.data ? me.data.request : null);
          // debug('request:',params);
          request(params, function (err, response) {
            debug('response:', err, response.statusCode);
            if (err || response.statusCode !== 200) {
              me.loadState = 3;
              me.err = err;
              me.response = response;
              reject(me);
            } else {
              try {
                mkdir(me.saveTo.substr(0, me.saveTo.lastIndexOf('/')));
                var hash = me.hash;
                var body = response.body;
                if (me.opts.isGBK && me.isText()) {
                  body = decode(body, 'gbk');
                }
                me.hash = Hash(body + '');
                me.response = response;
                me.loadState = 2;
                if (me.hash != hash) response.hasUpdated = true;
                if (me.opts.autoRelative && me.isHTML()) {
                  var result = convert.html(body + '', me.href);
                  debug('autoRelative', me.href, result.changed);
                  debug('autoRelative:', result.links);
                  body = result.html;
                }
                me.body = body;
                if (me.saveTo) {
                  me.emit('before_save', me);
                  fs.writeFileSync(me.saveTo, me.body);
                }
                resolve(me);
              } catch (e) {
                console.warning(e);
                reject(me);
              }
            }
          });
        } catch (e) {
          debug('load-error:', e);
          me.err = e;
          reject(me);
        }
      });
    }
    // 读取content

  }, {
    key: 'getBody',
    value: function getBody(encode) {
      if (this.body) return this.body;
      if (fs.exist(this.saveTo)) return fs.readFileSync(this.saveTo, encode);
      return false;
    }
  }, {
    key: 'isText',
    value: function isText() {
      if (this.response) {
        var contentType = this.response.headers['content-type'];
        debug('content-type:', contentType);
        if (/text|json|utf|gbk/.test(contentType)) return true;
        if (contentType.indexOf('image') >= 0) return false;
      }
      if (/(\.png|\.jpg|\.gif|\.woff|\.ico|\.ttf|\.eot)/.test(this.saveTo)) return false;
      if (/(\.htm|\.js|\.css|\.svg)/.test(this.saveTo)) return true;
      return false;
    }
  }, {
    key: 'isHTML',
    value: function isHTML() {
      if (this.response) {
        var contentType = this.response.headers['content-type'];
        debug('html-content-type:', contentType);
        if (/htm/.test(contentType)) return true;
      }
      if (/(\.htm)/.test(this.saveTo)) return true;
      return false;
    }
  }, {
    key: 'isCss',
    value: function isCss() {
      if (this.response) {
        var contentType = this.response.headers['content-type'];
        debug('html-content-type:', contentType);
        if (/css/.test(contentType)) return true;
      }
      if (/(\.css)/.test(this.saveTo)) return true;
      return false;
    }
  }, {
    key: 'isJs',
    value: function isJs() {
      if (this.response) {
        var contentType = this.response.headers['content-type'];
        debug('html-content-type:', contentType);
        if (/javascript/.test(contentType)) return true;
      }
      if (/(\.js)/.test(this.saveTo)) return true;
      return false;
    }
  }, {
    key: 'getLinks',
    value: function getLinks() {
      if (!this.isText()) return false;
      var body = this.getBody() + '';
      debug('links,body');
      if (body) {
        if (this.isHTML()) return _getLinks.html(body, this.href);
        if (this.isCss()) return _getLinks.css(body, this.href);
        return false;
      }
      return false;
    }
    // todo:保存文件
    // save(body, encode) {
    //
    // }
    /**
     * 是否有更新
     * @return [promise]
     */

  }, {
    key: 'hasUpdated',
    value: function hasUpdated() {
      return this.load().then(function (response) {
        if (response && response.hasUpdated) return response;
        throw response;
      });
    }
  }]);

  return File;
}(Event);

var headers = {
  // "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
  'Accept-Language': 'zh-CN,zh;q=0.8,en;q=0.6,zh-TW;q=0.4',
  'Cache-Control': 'no-cache',
  'Connection': 'keep-alive',
  'Pragma': 'no-cache',
  'Upgrade-Insecure-Requests': '1',
  'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'
};

module.exports = File;

==> spider-website-1.0.6.tgz: package/lib/file.js <==
'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * 文件对象
 *    新建对象
 *    从json中建立对象
 *    计算保存地址
 *    文件下载
 *    转换地址为相对地址
 *    触发事件
 *       init
 *       load_before
 *       loaded
 *       load_fail
 *       convert
 * */
var URL = require('url');
var isDir = require('is-dir');
var mkdir = require('mk-dir');
var request = require('request');
var fs = require('fs');
var Path = require('path');
var Hash = require('object-hash');
var _getLinks = require('ctx-links');
var convert = require('convert-relative');
var Event = require('events');
var debug = require('debug')('spider-file');
var decode = require('iconv-lite').decode;
var initState = {
  loaded: false,
  loadState: 0, // 下载状态，0：未开始，1：下载中，2： 下载完毕  3： 下载失败
  reTryTime: 0
};
var saveAttrs = ['href', 'loaded', 'data', 'reTryTime', 'hash', 'data', 'saveTo'];
var parseAttr = ['host', 'pathname', 'href', 'protocol'];

var defaultOpts = {};

var File = function (_Event) {
  _inherits(File, _Event);

  _createClass(File, null, [{
    key: 'setConfig',
    value: function setConfig(opts) {
      defaultOpts = opts;
    }
  }]);

  function File(urlORData, opts) {
    var _ret;

    _classCallCheck(this, File);

    debug('new file:', urlORData);

    var _this = _possibleConstructorReturn(this, (File.__proto__ || Object.getPrototypeOf(File)).call(this));

    _this.init(urlORData, Object.assign({}, opts, defaultOpts));
    debug('init file finished');
    return _ret = _this, _possibleConstructorReturn(_this, _ret);
  }

  _createClass(File, [{
    key: 'init',
    value: function init(data, opts) {
      if (!data) throw new Error('File.init(url,[opts]) nead url.');
      if (typeof data !== 'string' && typeof data.href !== 'string') throw new Error('File.init(url,[opts]) nead url.');
      var parse = URL.parse(typeof data === 'string' ? data : data.href);
      var me = this;
      parseAttr.forEach(function (attr) {
        me[attr] = parse[attr];
      });
      for (var attr in data) {
        if (saveAttrs.indexOf(attr) >= 0) me[attr] = data[attr] || initState[attr];
      }
      me.link = me.protocol + '//' + me.host + me.pathname;
      me.opts = opts || defaultOpts;
      if (!me.saveTo) me.saveTo = me.getSavePath();
      return me;
    }
  }, {
    key: 'toJSON',
    value: function toJSON() {
      var me = this;
      var obj = {};
      saveAttrs.forEach(function (attr) {
        obj[attr] = me[attr];
      });
      return JSON.stringify(obj);
    }
  }, {
    key: 'getSavePath',
    value: function getSavePath() {
      if (this.saveTo) return this.saveTo;
      // 判断是否需要增加扩展名
      var pathname = this.pathname || '';
      var extname = Path.extname(pathname);
      if (this.opts.autoName && (pathname[pathname.length - 1] === '/' || !extname)) {
        pathname += '/' + this.opts.autoName;
        extname = Path.extname(extname);
      }
      var index = this.opts.savePathIgnore && pathname.indexOf(this.opts.savePathIgnore);
      if (index === 0) pathname = pathname.substr(this.opts.savePathIgnore.length - 1);
      var saveTo = Path.resolve(this.opts.saveTo + '/' + pathname);
      if (isDir(saveTo)) throw new Error('saveTo path is directory...');
      return saveTo;
    }
  }, {
    key: 'load',
    value: function load() {
      var me = this;
      debug('load:', this.href);
      return new Promise(function (resolve, reject) {
        me.reTryTime = me.reTryTime + 1 || 1;
        me.loadState = 1;
        setTimeout(function () {
          debug('load timeout', me.opts.timeout || 100000);
          if (me.loadState == 1) {
            me.loadState = 3;
            me.err = new Error('timeout');
          }
          reject(me);
        }, me.opts.timeout || 100000);
        try {
          var params = Object.assign({
            url: encodeURI(me.href),
            method: 'get',
            gzip: false,
            headers: headers,
            encoding: null
          }, me.opts.request, me.data ? me.data.request : null);
          // debug('request:',params);
          request(params, function (err, response) {
            debug('response:', err, response.statusCode);
            if (err || response.statusCode !== 200) {
              me.loadState = 3;
              me.err = err;
              me.response = response;
              reject(me);
            } else {
              try {
                mkdir(me.saveTo.substr(0, me.saveTo.lastIndexOf('/')));
                var hash = me.hash;
                var body = response.body;
                if (me.opts.isGBK && me.isText()) {
                  body = decode(body, 'gbk');
                }
                me.hash = Hash(body + '');
                me.response = response;
                me.loadState = 2;
                if (me.hash != hash) response.hasUpdated = true;
                if (me.opts.autoRelative && me.isHTML()) {
                  var result = convert.html(body + '', me.href);
                  debug('autoRelative', me.href, result.changed);
                  debug('autoRelative:', result.links);
                  body = result.html;
                }
                me.body = body;
                if (me.saveTo) {
                  me.emit('before_save', me);
                  fs.writeFileSync(me.saveTo, me.body);
                }
                resolve(me);
              } catch (e) {
                console.warning(e);
                reject(me);
              }
            }
          });
        } catch (e) {
          debug('load-error:', e);
          me.err = e;
          reject(me);
        }
      });
    }
    // 读取content

  }, {
    key: 'getBody',
    value: function getBody(encode) {
      if (this.body) return this.body;
      if (fs.exist(this.saveTo)) return fs.readFileSync(this.saveTo, encode);
      return false;
    }
  }, {
    key: 'isText',
    value: function isText() {
      if (this.response) {
        var contentType = this.response.headers['content-type'];
        debug('content-type:', contentType);
        if (/text|json|utf|gbk/.test(contentType)) return true;
        if (contentType.indexOf('image') >= 0) return false;
      }
      if (/(\.png|\.jpg|\.gif|\.woff|\.ico|\.ttf|\.eot)/.test(this.saveTo)) return false;
      if (/(\.htm|\.js|\.css|\.svg)/.test(this.saveTo)) return true;
      return false;
    }
  }, {
    key: 'isHTML',
    value: function isHTML() {
      if (this.response) {
        var contentType = this.response.headers['content-type'];
        debug('html-content-type:', contentType);
        if (/htm/.test(contentType)) return true;
      }
      if (/(\.htm)/.test(this.saveTo)) return true;
      return false;
    }
  }, {
    key: 'isCss',
    value: function isCss() {
      if (this.response) {
        var contentType = this.response.headers['content-type'];
        debug('html-content-type:', contentType);
        if (/css/.test(contentType)) return true;
      }
      if (/(\.css)/.test(this.saveTo)) return true;
      return false;
    }
  }, {
    key: 'isJs',
    value: function isJs() {
      if (this.response) {
        var contentType = this.response.headers['content-type'];
        debug('html-content-type:', contentType);
        if (/javascript/.test(contentType)) return true;
      }
      if (/(\.js)/.test(this.saveTo)) return true;
      return false;
    }
  }, {
    key: 'getLinks',
    value: function getLinks() {
      if (!this.isText()) return false;
      var body = this.getBody() + '';
      debug('links,body');
      if (body) {
        if (this.isHTML()) return _getLinks.html(body, this.href);
        if (this.isCss()) return _getLinks.css(body, this.href);
        return false;
      }
      return false;
    }
    // todo:保存文件
    // save(body, encode) {
    //
    // }
    /**
     * 是否有更新
     * @return [promise]
     */

  }, {
    key: 'hasUpdated',
    value: function hasUpdated() {
      return this.load().then(function (response) {
        if (response && response.hasUpdated) return response;
        throw response;
      });
    }
  }]);

  return File;
}(Event);

var headers = {
  // "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
  'Accept-Language': 'zh-CN,zh;q=0.8,en;q=0.6,zh-TW;q=0.4',
  'Cache-Control': 'no-cache',
  'Connection': 'keep-alive',
  'Pragma': 'no-cache',
  'Upgrade-Insecure-Requests': '1',
  'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'
};

module.exports = File;
